@@ -8,6 +8,7 @@ import time
 import hashlib
 import json
 import time
+import re
 
 import logging
 import logging.handlers
@@ -17,7 +18,7 @@ import evalidate
 import socket
 from socket import AF_INET, SOCK_STREAM, SOCK_DGRAM
 
-version='2.0.15 (daemon)'
+version='2.0.48 (traceback)'
 
 
 # TODO
@@ -80,6 +81,8 @@ class OkerrClient:
         self.retry = False # default
         self.sleeptime = 10
         
+        self.dry_run = False 
+        
         if cfg:            
             for k in self.cfgvars:
                 if k in cfg:
@@ -100,6 +103,8 @@ class OkerrClient:
                 textid=self.textid, 
                 secret=self.secret)    
 
+    def dry(self,dry=True):
+        self.dry_run = dry
 
     def load(self):
         self.loadcache()
@@ -159,7 +164,7 @@ class OkerrClient:
                 pass
 
         # if we are here, then no auth problem, but no key
-        raise OkerrNoKey('Failed all {} keypath: {}'.format(ntried, path))
+        raise OkerrNoKey('Failed all {} keypath: {} (textid: {})'.format(ntried, path, self.textid))
 
     def keypath(self,path):
 
@@ -173,7 +178,7 @@ class OkerrClient:
         url = self.url+'getkeyval/{}/{}'.format(self.textid,path)
 
         auth=None
-        
+                
         if self.keyuser and self.keypath:
             auth=(self.keyuser,self.keypass)
                 
@@ -205,6 +210,10 @@ class OkerrClient:
     def update(self,name,status,details=None,
         method=None,tags=None,error=None, origkeypath=None, keypath=None):
         
+        if self.dry_run:
+            self.log.debug('Do NOT update: dry run. {} = {}'.format(name, repr(status)))            
+            return
+        
         if not self.textid:
             self.log.error('Do not update: no textid')
             return
@@ -246,10 +255,14 @@ class OkerrClient:
         start = time.time()
         
         
-        if len(str(status))>20:
-            preview = str(status)[:18]+".."
+        preview = str(status)
+        
+        preview = re.sub('[\r\n]'," ", preview)
+        
+        if len(str(preview))>40:
+            preview = str(preview)[:38]+".."
         else:
-            preview = str(status)
+            preview = str(preview)
             
         
         stop = False
@@ -265,10 +278,11 @@ class OkerrClient:
                         format(name,preview))
                 else:
                     if self.retry:
+                        # not daemon mode, retry=false in daemon
                         self.log.error('okerr update error ({}) textid:{}, {}={} {}'.\
-                            format(r.status_code,self.textid,name,status,secretlog))            
+                            format(r.status_code,self.textid,name,preview,secretlog))            
                     else:
-                        raise OkerrClientServer('okerr update error {} \'{:.50}\' textid:{}, {}={} {}'.\
+                        raise OkerrClientServer('okerr update error {} \'{:.150}\' textid:{}, {}={} {}'.\
                             format(r.status_code,briefpage(r.text),self.textid,name,status,secretlog))
             
                 self.log.debug("Request to URL {}:".format(r.request.url))
