@@ -74,13 +74,15 @@ class TaskProcesses(ProcTaskProcessor):
     def run(self,ts,data,args):
         outdata = []
         count = 0
-        attrs = ['name','exe','username','pid']
+        attrs = ['name','exe','username','pid','create_time']
         for p in psutil.process_iter():
             ps={}
             for attr in attrs:
                 ps = p.as_dict(attrs=attrs)
-                if 'exe' in ps and ps['exe']:
-                    ps['basename']=os.path.basename(ps['exe'])
+            if 'exe' in ps and ps['exe']:
+                ps['basename']=os.path.basename(ps['exe'])
+            ps['agesec'] = int( time.time() - ps['create_time'] )
+                
 
             outdata.append(ps)
                                    
