@@ -17,7 +17,7 @@ from operator import itemgetter
 
 import okerrclient
 from okerrclient import OkerrExc, OkerrNoKey, OkerrNoTextID, OkerrKeyAuth, \
-    OkerrBadData, OkerrBadMethod
+    OkerrBadData, OkerrBadMethod, OkerrClientServer
 
 #
 # Taskprocessor is parent class for processor, which implements sequence code
@@ -145,6 +145,7 @@ class TaskSeq():
     iname = None # indicator
     method = None # method of indicator
     details = None # details
+    except_data = None # used for exception
 
 
     #
@@ -300,6 +301,7 @@ class TaskSeq():
         ts.oc = self.oc        
         ts.method = method
         ts.details = self.details
+        ts.except_data = self.except_data
         ts.parent = self
         ts.tags = list(self.tags)
         ts.keypath = self.keypath
@@ -350,6 +352,8 @@ class TaskSeq():
             # method is not child of TaskProcessor
             raise BadMethodException()
             
+            
+    # taskseq.help            
     @classmethod
     def help(cls):
 
@@ -417,7 +421,8 @@ class TaskSeq():
     
             if self._stop:
                 self.oc.log.debug('sequence name:{} id:{} got stop signal'.format(self.iname, id(self)))
-                return
+                data = self.except_data
+                break
 
             r = r.copy()
 
@@ -429,11 +434,17 @@ class TaskSeq():
             
             if isinstance(data,six.string_types):
                 vv['_str'] = data
+
+            if isinstance(data,dict):
+                for k,v in data.items():
+                    varname = "_dict_{}".format(k)
+                    vv[varname]=v
+
                            
             # replace vars
             for argname,argval in r.items():
-                # print("replace arg {} = {}".format(argname, argval))
-                # print(vv)
+                #print("replace arg {} = {}".format(argname, argval))
+                #print(vv)
                 newval = string.Template(argval).safe_substitute(vv)
                 #print("newval: {}".format(newval))
                 r[argname] = newval
@@ -453,9 +464,12 @@ class TaskSeq():
         
         if self.iname is not None: 
             if data is not None:
-                       
-                self.oc.update(self.iname,data,details=self.details,
-                method=self.method,tags=self.tags,error=None, origkeypath = ':'.join(self.origkeypath), keypath = ':'.join(self.keypath))
+                try:
+                    self.oc.update(self.iname,data,details=self.details,
+                        method=self.method,tags=self.tags,error=None, origkeypath = ':'.join(self.origkeypath), 
+                        keypath = ':'.join(self.keypath))
+                except OkerrExc as e:
+                    self.oc.log.error("update failed: {}".format(str(e)))
             else:
                 self.oc.log.debug('do no update indicator {}, because data is None'.format(self.iname))
         else:
@@ -480,9 +494,7 @@ class TaskAlias(TaskProcessor):
         return msg
     
     def run(self,ts,data,args):
-        print("ALIAS RUN")
         for code in self.alias:
-            print("RUN CODE {}".format(code))
             data = ts.tp[code].run(ts,data,args)
 
         return data
@@ -512,6 +524,21 @@ TaskMyName('ERR',TaskSeq)
 
 
 
+class TaskExcept(GenTaskProcessor):
+    help = 'Data to use in case or exception'
+
+    parse_args=False
+    store_argline='data'
+    
+    defargs= { 'data': ''}
+
+    def run(self,ts,data,args):
+        ts.except_data = args['data']
+        return data
+    
+TaskExcept('EXCEPT',TaskSeq)
+
+
 class TaskSet(GenTaskProcessor):
     
     parse_args=False
@@ -721,14 +748,13 @@ TaskInt('INT',TaskSeq)
 
 class TaskNonZero(GenTaskProcessor):
     help = 'Returns OK if input is number higher then 0, or returns ERR otherwise'
-    def run(self,ts,data,args):
-        print("nonzero data:",data)
+    def run(self,ts,data,args):        
         try:
-            if int(data)>0:
+            if int(float(data))>0:
                 return 'OK'
             else:
                 return 'ERR'
-        except ValueError:
+        except (ValueError, TypeError):
             ts.oc.log.error('bad data for {}: {} (must be numerical)'.format(self.code, data))
             return 'ERR'
 
@@ -751,7 +777,7 @@ class TaskNone(GenTaskProcessor):
 TaskNone('NONE',TaskSeq)
 
 class TaskSort(GenTaskProcessor):
-    help = 'sort words or lines or lists'
+    help = 'sort words or lines in string, or lists or lists of dicts'
     defargs = {'num': '0', 'sep': ' ', 'empty': '0','field': ''}
         
     def run(self,ts,data,args):
@@ -762,7 +788,7 @@ class TaskSort(GenTaskProcessor):
             raise OkerrBadData('attempt to SORT None data')
     
         # make sure we have list
-        if isinstance(data,str):
+        if isinstance(data,six.string_types):
             was_str=True
             data = data.split(sep)
         
@@ -813,6 +839,7 @@ class TaskSeqDump(GenTaskProcessor):
             print ("\t{} ".format(TaskProcessor.cmd2str(r)))
             
         print("details:",ts.details)
+        print("except:",ts.except_data)        
         print("method:",ts.method)
         print("tags:",ts.tags)
         print("variables:",json.dumps(ts.v, sort_keys=True, indent=4, separators=(',',': ')))
@@ -941,12 +968,14 @@ class TaskMkSeq(GenTaskProcessor):
         if data is None :
             # print("no data")
             if args['path']:
-                data,path = ts.oc.altkeypath(args['path'])
-                # everything is fine!
-                self.origkeypath = path.split(':')                
-                self.keypath=list()
-                    
-                                    
+                try:
+                    data,path = ts.oc.altkeypath(args['path'])
+                    # everything is fine!
+                    self.origkeypath = path.split(':')                
+                    self.keypath=list()
+                except OkerrExc as e:
+                    ts.oc.log.error('get keys error: {}'.format(str(e)))
+                    return None                
                     
             else:
                 ts.oc.log.debug('will not MKSEQ because data is None and no path given (or empty)')
@@ -1130,14 +1159,14 @@ class TaskKMGT(FormatTaskProcessor):
     help = 'Convert number (e.g. size of file) to string with suffix (e.g. 12M)'
 
     defargs = {
-        'frac': '10',
+        'frac': '1',
         'field': '',
         'destfield': ''
     }
 
 
     @staticmethod
-    def kmgt_short(sz, frac=10):
+    def kmgt_short(sz, frac=1):
         t={
             'K': pow(1024,1),
             'M': pow(1024,2),
@@ -1146,9 +1175,16 @@ class TaskKMGT(FormatTaskProcessor):
             '': 1}
             
         for k in sorted(t,key=t.__getitem__,reverse=True):
-            if sz>t[k]:
-                n = int((float(sz)*frac / t[k]))/float(frac)
-                return "{}{}".format(n,k)
+            fmul = pow(10,frac)
+        
+            if sz>=t[k]:
+                #n = int((float(sz)*fmul / t[k]))
+                n = sz/float(t[k])                
+                #n = n/float(fmul)
+                
+                tpl = "{:."+str(frac)+"f}{}"
+                
+                return tpl.format(n,k)
 
     def run(self,ts,data,args):               
 
